/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 *
 * Command-line application for printing MolViewSpec tree schema
 * Build: npm run build
 * Run:   node lib/commonjs/cli/mvs/mvs-print-schema
 *        node lib/commonjs/cli/mvs/mvs-print-schema --markdown
 */
import { __assign } from "tslib";
import { ArgumentParser } from 'argparse';
import { treeSchemaToMarkdown, treeSchemaToString } from '../../extensions/mvs/tree/generic/tree-schema';
import { MVSDefaults } from '../../extensions/mvs/tree/mvs/mvs-defaults';
import { MVSTreeSchema } from '../../extensions/mvs/tree/mvs/mvs-tree';
/** Return parsed command line arguments for `main` */
function parseArguments() {
    var parser = new ArgumentParser({ description: 'Command-line application for printing MolViewSpec tree schema.' });
    parser.add_argument('-m', '--markdown', { action: 'store_true', help: 'Print the schema as markdown instead of plain text.' });
    var args = parser.parse_args();
    return __assign({}, args);
}
/** Main workflow for printing MolViewSpec tree schema. */
function main(args) {
    if (args.markdown) {
        console.log(treeSchemaToMarkdown(MVSTreeSchema, MVSDefaults));
    }
    else {
        console.log(treeSchemaToString(MVSTreeSchema, MVSDefaults));
    }
}
main(parseArguments());
