/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 *
 * Command-line application for validating MolViewSpec files
 * Build: npm run build
 * Run:   node lib/commonjs/cli/mvs/mvs-validate examples/mvs/1cbs.mvsj
 */
import { __assign } from "tslib";
import { ArgumentParser } from 'argparse';
import fs from 'fs';
import { setFSModule } from '../../mol-util/data-source';
import { MVSData } from '../../extensions/mvs/mvs-data';
setFSModule(fs);
/** Return parsed command line arguments for `main` */
function parseArguments() {
    var parser = new ArgumentParser({ description: 'Command-line application for validating MolViewSpec files. Prints validation status (OK/FAILED) to stdout, detailed validation issues to stderr. Exits with a zero exit code if all input files are OK.' });
    parser.add_argument('input', { nargs: '+', help: 'Input file(s) in .mvsj format' });
    parser.add_argument('--no-extra', { action: 'store_true', help: 'Treat presence of extra node params as an issue.' });
    var args = parser.parse_args();
    return __assign({}, args);
}
/** Main workflow for validating MolViewSpec files. Returns the number of failed input files. */
function main(args) {
    var nFailed = 0;
    for (var _i = 0, _a = args.input; _i < _a.length; _i++) {
        var input = _a[_i];
        var data = fs.readFileSync(input, { encoding: 'utf8' });
        var mvsData = MVSData.fromMVSJ(data);
        var issues = MVSData.validationIssues(mvsData, { noExtra: args.no_extra });
        var status_1 = issues ? 'FAILED' : 'OK';
        console.log("".concat(status_1.padEnd(6), " ").concat(input));
        if (issues) {
            nFailed++;
            for (var _b = 0, issues_1 = issues; _b < issues_1.length; _b++) {
                var issue = issues_1[_b];
                console.error(issue);
            }
        }
    }
    return nFailed;
}
var nFailed = main(parseArguments());
if (nFailed > 0) {
    process.exitCode = 1;
}
