"use strict";
/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 *
 * Command-line application for printing MolViewSpec tree schema
 * Build: npm run build
 * Run:   node lib/commonjs/cli/mvs/mvs-print-schema
 *        node lib/commonjs/cli/mvs/mvs-print-schema --markdown
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var argparse_1 = require("argparse");
var tree_schema_1 = require("../../extensions/mvs/tree/generic/tree-schema");
var mvs_defaults_1 = require("../../extensions/mvs/tree/mvs/mvs-defaults");
var mvs_tree_1 = require("../../extensions/mvs/tree/mvs/mvs-tree");
/** Return parsed command line arguments for `main` */
function parseArguments() {
    var parser = new argparse_1.ArgumentParser({ description: 'Command-line application for printing MolViewSpec tree schema.' });
    parser.add_argument('-m', '--markdown', { action: 'store_true', help: 'Print the schema as markdown instead of plain text.' });
    var args = parser.parse_args();
    return tslib_1.__assign({}, args);
}
/** Main workflow for printing MolViewSpec tree schema. */
function main(args) {
    if (args.markdown) {
        console.log((0, tree_schema_1.treeSchemaToMarkdown)(mvs_tree_1.MVSTreeSchema, mvs_defaults_1.MVSDefaults));
    }
    else {
        console.log((0, tree_schema_1.treeSchemaToString)(mvs_tree_1.MVSTreeSchema, mvs_defaults_1.MVSDefaults));
    }
}
main(parseArguments());
