"use strict";
/**
 * Copyright (c) 2021 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Mandar Deshpande <mandar@ebi.ac.uk>
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PLDDTConfidenceColorThemeProvider = exports.PLDDTConfidenceColorTheme = exports.getPLDDTConfidenceColorThemeParams = void 0;
var tslib_1 = require("tslib");
var prop_1 = require("../prop");
var structure_1 = require("../../../../mol-model/structure");
var color_1 = require("../../../../mol-theme/color");
var color_2 = require("../../../../mol-util/color");
var param_definition_1 = require("../../../../mol-util/param-definition");
var legend_1 = require("../../../../mol-util/legend");
var DefaultColor = (0, color_2.Color)(0xaaaaaa);
var ConfidenceColors = {
    'No Score': DefaultColor,
    'Very Low': (0, color_2.Color)(0xff7d45),
    'Low': (0, color_2.Color)(0xffdb13),
    'Confident': (0, color_2.Color)(0x65cbf3),
    'Very High': (0, color_2.Color)(0x0053d6)
};
var ConfidenceColorLegend = (0, legend_1.TableLegend)(Object.entries(ConfidenceColors));
function getPLDDTConfidenceColorThemeParams(ctx) {
    return {};
}
exports.getPLDDTConfidenceColorThemeParams = getPLDDTConfidenceColorThemeParams;
function PLDDTConfidenceColorTheme(ctx, props) {
    var color = function () { return DefaultColor; };
    if (ctx.structure) {
        var l_1 = structure_1.StructureElement.Location.create(ctx.structure.root);
        var getColor_1 = function (location) {
            var _a, _b;
            var unit = location.unit, element = location.element;
            if (!structure_1.Unit.isAtomic(unit))
                return DefaultColor;
            var qualityAssessment = prop_1.QualityAssessmentProvider.get(unit.model).value;
            var score = (_b = (_a = qualityAssessment === null || qualityAssessment === void 0 ? void 0 : qualityAssessment.pLDDT) === null || _a === void 0 ? void 0 : _a.get(unit.model.atomicHierarchy.residueAtomSegments.index[element])) !== null && _b !== void 0 ? _b : -1;
            if (score < 0) {
                return DefaultColor;
            }
            else if (score <= 50) {
                return (0, color_2.Color)(0xff7d45);
            }
            else if (score <= 70) {
                return (0, color_2.Color)(0xffdb13);
            }
            else if (score <= 90) {
                return (0, color_2.Color)(0x65cbf3);
            }
            else {
                return (0, color_2.Color)(0x0053d6);
            }
        };
        color = function (location) {
            if (structure_1.StructureElement.Location.is(location)) {
                return getColor_1(location);
            }
            else if (structure_1.Bond.isLocation(location)) {
                l_1.unit = location.aUnit;
                l_1.element = location.aUnit.elements[location.aIndex];
                return getColor_1(l_1);
            }
            return DefaultColor;
        };
    }
    return {
        factory: PLDDTConfidenceColorTheme,
        granularity: 'group',
        preferSmoothing: true,
        color: color,
        props: props,
        description: 'Assigns residue colors according to the pLDDT Confidence score.',
        legend: ConfidenceColorLegend
    };
}
exports.PLDDTConfidenceColorTheme = PLDDTConfidenceColorTheme;
exports.PLDDTConfidenceColorThemeProvider = {
    name: 'plddt-confidence',
    label: 'pLDDT Confidence',
    category: color_1.ColorTheme.Category.Validation,
    factory: PLDDTConfidenceColorTheme,
    getParams: getPLDDTConfidenceColorThemeParams,
    defaultValues: param_definition_1.ParamDefinition.getDefaultValues(getPLDDTConfidenceColorThemeParams({})),
    isApplicable: function (ctx) { var _a; return !!((_a = ctx.structure) === null || _a === void 0 ? void 0 : _a.models.some(function (m) { return prop_1.QualityAssessment.isApplicable(m, 'pLDDT'); })); },
    ensureCustomProperties: {
        attach: function (ctx, data) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var _i, _a, m;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!data.structure) return [3 /*break*/, 4];
                        _i = 0, _a = data.structure.models;
                        _b.label = 1;
                    case 1:
                        if (!(_i < _a.length)) return [3 /*break*/, 4];
                        m = _a[_i];
                        return [4 /*yield*/, prop_1.QualityAssessmentProvider.attach(ctx, m, void 0, true)];
                    case 2:
                        _b.sent();
                        _b.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        }); },
        detach: function (data) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var _i, _a, m;
            return tslib_1.__generator(this, function (_b) {
                if (data.structure) {
                    for (_i = 0, _a = data.structure.models; _i < _a.length; _i++) {
                        m = _a[_i];
                        prop_1.QualityAssessmentProvider.ref(m, false);
                    }
                }
                return [2 /*return*/];
            });
        }); }
    }
};
