"use strict";
/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.textPropsForSelection = void 0;
var boundary_helper_1 = require("../../../mol-math/geometry/boundary-helper");
var linear_algebra_1 = require("../../../mol-math/linear-algebra");
var structure_1 = require("../../../mol-model/structure");
var array_1 = require("../../../mol-util/array");
var atom_ranges_1 = require("./atom-ranges");
var indexing_1 = require("./indexing");
var selections_1 = require("./selections");
var tmpVec = (0, linear_algebra_1.Vec3)();
var tmpArray = [];
var boundaryHelper = new boundary_helper_1.BoundaryHelper('98');
var outAtoms = [];
var outFirstAtomIndex = {};
/** Return `TextProps` (position, size, etc.) for a text that is to be bound to a substructure of `structure` defined by union of `rows`.
 * Derives `center` and `depth` from the boundary sphere of the substructure, `scale` from the number of heavy atoms in the substructure. */
function textPropsForSelection(structure, sizeFunction, rows, onlyInModel) {
    var _a;
    var _b;
    var loc = structure_1.StructureElement.Location.create(structure);
    var units = structure.units;
    var type_symbol = structure_1.StructureProperties.atom.type_symbol;
    tmpArray.length = 0;
    var includedAtoms = 0;
    var includedHeavyAtoms = 0;
    var group = undefined;
    var atomSize = undefined;
    var rangesByModel = {};
    for (var iUnit = 0, nUnits = units.length; iUnit < nUnits; iUnit++) {
        var unit = units[iUnit];
        if (onlyInModel && unit.model.id !== onlyInModel.id)
            continue;
        var ranges = (_a = rangesByModel[_b = unit.model.id]) !== null && _a !== void 0 ? _a : (rangesByModel[_b] = (0, selections_1.getAtomRangesForRows)(unit.model, rows, indexing_1.IndicesAndSortings.get(unit.model)));
        var position = unit.conformation.position;
        loc.unit = unit;
        atom_ranges_1.AtomRanges.selectAtomsInRanges(unit.elements, ranges, outAtoms, outFirstAtomIndex);
        for (var _i = 0, outAtoms_1 = outAtoms; _i < outAtoms_1.length; _i++) {
            var atom = outAtoms_1[_i];
            loc.element = atom;
            position(atom, tmpVec);
            (0, array_1.arrayExtend)(tmpArray, tmpVec);
            group !== null && group !== void 0 ? group : (group = structure.serialMapping.cumulativeUnitElementCount[iUnit] + outFirstAtomIndex.value);
            atomSize !== null && atomSize !== void 0 ? atomSize : (atomSize = sizeFunction(loc));
            includedAtoms++;
            if (type_symbol(loc) !== 'H')
                includedHeavyAtoms++;
        }
    }
    if (includedAtoms > 0) {
        var _c = (includedAtoms > 1) ? boundarySphere(tmpArray) : { center: linear_algebra_1.Vec3.fromArray((0, linear_algebra_1.Vec3)(), tmpArray, 0), radius: 1.1 * atomSize }, center = _c.center, radius = _c.radius;
        var scale = Math.pow((includedHeavyAtoms || includedAtoms), (1 / 3));
        return { center: center, depth: radius, scale: scale, group: group };
    }
}
exports.textPropsForSelection = textPropsForSelection;
/** Calculate the boundary sphere for a set of points given by their flattened coordinates (`flatCoords.slice(0,3)` is the first point etc.) */
function boundarySphere(flatCoords) {
    var length = flatCoords.length;
    boundaryHelper.reset();
    for (var offset = 0; offset < length; offset += 3) {
        linear_algebra_1.Vec3.fromArray(tmpVec, flatCoords, offset);
        boundaryHelper.includePosition(tmpVec);
    }
    boundaryHelper.finishedIncludeStep();
    for (var offset = 0; offset < length; offset += 3) {
        linear_algebra_1.Vec3.fromArray(tmpVec, flatCoords, offset);
        boundaryHelper.radiusPosition(tmpVec);
    }
    return boundaryHelper.getSphere();
}
