/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 */
import { __assign } from "tslib";
import { TextBuilder } from '../../../../mol-geo/geometry/text/text-builder';
import { ComplexTextVisual } from '../../../../mol-repr/structure/complex-visual';
import * as Original from '../../../../mol-repr/structure/visual/label-text';
import { ElementIterator, eachSerialElement, getSerialElementLoci } from '../../../../mol-repr/structure/visual/util/element';
import { ColorNames } from '../../../../mol-util/color/names';
import { omitObjectKeys } from '../../../../mol-util/object';
import { ParamDefinition as PD } from '../../../../mol-util/param-definition';
import { textPropsForSelection } from '../../helpers/label-text';
import { groupRows } from '../../helpers/selections';
import { getMVSAnnotationForStructure } from '../annotation-prop';
export var MVSAnnotationLabelTextParams = __assign(__assign({ annotationId: PD.Text('', { description: 'Reference to "Annotation" custom model property', isEssential: true }), fieldName: PD.Text('label', { description: 'Annotation field (column) from which to take label contents', isEssential: true }) }, omitObjectKeys(Original.LabelTextParams, ['level', 'chainScale', 'residueScale', 'elementScale'])), { borderColor: __assign(__assign({}, Original.LabelTextParams.borderColor), { defaultValue: ColorNames.black }) });
/** Create "label-text" visual for "MVS Annotation Label" representation */
export function MVSAnnotationLabelTextVisual(materialId) {
    return ComplexTextVisual({
        defaultProps: PD.getDefaultValues(MVSAnnotationLabelTextParams),
        createGeometry: createLabelText,
        createLocationIterator: ElementIterator.fromStructure,
        getLoci: getSerialElementLoci,
        eachLocation: eachSerialElement,
        setUpdateState: function (state, newProps, currentProps) {
            state.createGeometry = newProps.annotationId !== currentProps.annotationId || newProps.fieldName !== currentProps.fieldName;
        }
    }, materialId);
}
function createLabelText(ctx, structure, theme, props, text) {
    var _a;
    var _b = getMVSAnnotationForStructure(structure, props.annotationId), annotation = _b.annotation, model = _b.model;
    var rows = (_a = annotation === null || annotation === void 0 ? void 0 : annotation.getRows()) !== null && _a !== void 0 ? _a : [];
    var _c = groupRows(rows), count = _c.count, offsets = _c.offsets, grouped = _c.grouped;
    var builder = TextBuilder.create(props, count, count / 2, text);
    for (var iGroup = 0; iGroup < count; iGroup++) {
        var iFirstRowInGroup = grouped[offsets[iGroup]];
        var labelText = annotation.getValueForRow(iFirstRowInGroup, props.fieldName);
        if (!labelText)
            continue;
        var rowsInGroup = grouped.slice(offsets[iGroup], offsets[iGroup + 1]).map(function (j) { return rows[j]; });
        var p = textPropsForSelection(structure, theme.size.size, rowsInGroup, model);
        if (!p)
            continue;
        builder.add(labelText, p.center[0], p.center[1], p.center[2], p.depth, p.scale, p.group);
    }
    return builder.getText();
}
