/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 */
import { DataFormatProvider } from '../../../mol-plugin-state/formats/provider';
import { PluginStateObject as SO } from '../../../mol-plugin-state/objects';
import { StateAction, StateObjectRef } from '../../../mol-state';
import { ParamDefinition as PD } from '../../../mol-util/param-definition';
import { MVSData } from '../mvs-data';
declare const Mvs_base: {
    new (data: {
        mvsData: MVSData;
        sourceUrl?: string | undefined;
    }, props?: {
        label: string;
        description?: string | undefined;
    } | undefined): {
        id: import("../../../mol-util/uuid").UUID;
        type: SO.TypeInfo;
        label: string;
        description?: string | undefined;
        data: {
            mvsData: MVSData;
            sourceUrl?: string | undefined;
        };
    };
    type: SO.TypeInfo;
    is(obj?: import("../../../mol-state/object").StateObject<any, import("../../../mol-state/object").StateObject.Type<any>> | undefined): obj is import("../../../mol-state/object").StateObject<{
        mvsData: MVSData;
        sourceUrl?: string | undefined;
    }, SO.TypeInfo>;
};
/** Plugin state object storing `MVSData` */
export declare class Mvs extends Mvs_base {
}
/** Transformer for parsing data in MVSJ format */
export declare const ParseMVSJ: import("../../../mol-state/transformer").StateTransformer<SO.Data.String, Mvs, PD.Normalize<{}>>;
/** State action which loads a MVS view into Mol* */
export declare const LoadMvsData: StateAction<Mvs, void, PD.Normalize<{
    replaceExisting: boolean;
}>>;
/** Data format provider for MVSJ format.
 * If Visuals:On, it will load the parsed MVS view;
 * otherwise it will just create a plugin state object with parsed data. */
export declare const MVSJFormatProvider: DataFormatProvider<{}, StateObjectRef<Mvs>, any>;
export {};
