/**
 * Copyright (c) 2019 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author David Sehnal <david.sehnal@gmail.com>
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 */
import { __awaiter, __generator } from "tslib";
import * as File from '../common/file';
import { Ccp4Provider } from './format/ccp4';
import { getElementByteSize, createTypedArrayBufferContext } from '../../../mol-io/common/typed-array';
import { Dsn6Provider } from './format/dsn6';
import { fileHandleFromDescriptor } from '../../common/file-handle';
export function assignSliceBuffer(data, blockSizeInMB) {
    var _a = data.header, extent = _a.extent, valueType = _a.valueType;
    var maxBlockBytes = blockSizeInMB * 1024 * 1024;
    var sliceSize = extent[0] * extent[1] * getElementByteSize(valueType);
    var sliceCapacity = Math.max(1, Math.floor(Math.min(maxBlockBytes, sliceSize * extent[2]) / sliceSize));
    var buffer = createTypedArrayBufferContext(sliceCapacity * extent[0] * extent[1], valueType);
    data.slices = {
        buffer: buffer,
        maxBlockBytes: maxBlockBytes,
        sliceCapacity: sliceCapacity,
        slicesRead: 0,
        values: buffer.values,
        sliceCount: 0,
        isFinished: false
    };
}
function compareProp(a, b) {
    if (a instanceof Array && b instanceof Array) {
        if (a.length !== b.length)
            return false;
        for (var i = 0; i < a.length; i++) {
            if (a[i] !== b[i])
                return false;
        }
        return true;
    }
    return a === b;
}
export function compareHeaders(a, b) {
    for (var _i = 0, _a = ['grid', 'axisOrder', 'extent', 'origin', 'spacegroupNumber', 'cellSize', 'cellAngles', 'mode']; _i < _a.length; _i++) {
        var p = _a[_i];
        if (!compareProp(a[p], b[p]))
            return false;
    }
    return true;
}
export function getProviderFromType(type) {
    switch (type) {
        case 'ccp4': return Ccp4Provider;
        case 'dsn6': return Dsn6Provider;
    }
}
export function open(name, filename, type) {
    return __awaiter(this, void 0, void 0, function () {
        var provider, descriptor, file, header, data;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    provider = getProviderFromType(type);
                    return [4 /*yield*/, File.openRead(filename)];
                case 1:
                    descriptor = _a.sent();
                    file = fileHandleFromDescriptor(descriptor, filename);
                    return [4 /*yield*/, provider.readHeader(name, file)];
                case 2:
                    header = _a.sent();
                    data = { header: header, file: file, slices: void 0 };
                    return [2 /*return*/, { data: data, provider: provider }];
            }
        });
    });
}
